var Sequencing = function()
{
	var seq = {};
	var debug = 0;

	var flowCellIdIsValid = false;
	var sampleSheetIsValid = false;
	var autoAnalyzeIsValid = false;
	
	var checkedLibraries;

	// Page initialization
	seq.initPage = function()
	{
		// Step 1
		Events.addEventHandler('flowCellId', 'blur', seq.flowCellIdOnChange);
		Events.addEventHandler('sampleSheetFile', 'change', seq.sampleSheetFileOnChange);
		Events.addEventHandler('startDate', 'change', Wizard.validateDate);

		Events.addEventHandler('step-1', 'wizard-validate', seq.validateStep1);

		Events.addEventHandler('autoAnalyzeYes', 'click', seq.autoAnalyzeOnChange);
		Events.addEventHandler('autoAnalyzeNo', 'click', seq.autoAnalyzeOnChange);
		Events.addEventHandler('clusters', 'change', seq.clusterOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', seq.submit);

		Doc.show('step-1');
		Doc.show('goregister');
		Doc.element('flowCellId').focus();

		// Load Library protocols
		Reggie.loadProtocols('LIBRARY_PROTOCOL', 'libProtocol', 'LIBPREP_TARGET', 'External');
		
		var frm = document.forms['reggie'];
		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, seq.clustersLoaded);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		frm.debug.checked = debug || location.protocol != 'https:';
		
		if (debug)
		{
			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryPlatesForLibPrep&plateType=EXTERNAL_LIBRARY';
			Wizard.asyncJsonRequest(url, seq.initDebugTools);
		}

	}

	
	seq.flowCellIdOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		
		Wizard.setInputStatus('flowCellId');
		flowCellIdIsValid = false;
		
		var id = frm.flowCellId.value;
		if (id == '')
		{
			Wizard.setInputStatus('flowCellId', 'invalid', 'ID is a required value');
			return;
		}
		
		// Check if ID already exists in DB
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlowCellByBarcode';
		url += '&barcode='+encodeURIComponent(id);
		Wizard.showLoadingAnimation('Checking flow cell id...');
		Wizard.asyncJsonRequest(url, seq.flowCellIdChecked);
	}
	
	seq.flowCellIdChecked = function(response)
	{
		var frm = document.forms['reggie'];
		var existing = response.flowCell;
		if (existing != null)
		{
			var id = frm.flowCellId.value;
			Wizard.setInputStatus('flowCellId', 'invalid', 'Flow cell with ID='+Strings.encodeTags(id)+' already exists: ' + Strings.encodeTags(existing.name));
			return;
		}
		Wizard.setInputStatus('flowCellId', 'valid');
		flowCellIdIsValid = true;
	}

	
	seq.sampleSheetFileOnChange = function()
	{
		var frm = document.forms['reggie'];
		var file = frm.sampleSheetFile.files[0];
		
		Wizard.hideWizardStatus();
		Wizard.setInputStatus('sampleSheetFile');
		sampleSheetIsValid = false;
		
		var reader = new FileReader();
		reader.onload = seq.sampleSheetFileLoaded;
		reader.readAsText(file);
	}

	seq.sampleSheetFileLoaded = function(event)
	{
		seq.parseSampleSheet(event.target.result);
	}

	seq.parseSampleSheet = function(data)
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('sampleSheetFile');
		sampleSheetIsValid = false;

		var lines = data.split(/[\n\r]+/);
		var lineNo = 0;
		
		// Parse out some headers
		while (lineNo < lines.length) 
		{
			var line = lines[lineNo];
			lineNo++;
			
			if (line.indexOf('[Data]') >= 0) break;	// Break when we reach the [Data] section
			
			var columns = line.split(/,/);
			if (columns[0] == 'Date')
			{
				var startDate = Dates.parseString(columns[1], 'M/d/y');
				Doc.element('startDate').value = startDate == null ? columns[1] : Dates.formatDate(startDate, 'yyyyMMdd');
				Events.sendChangeEvent('startDate');
			}
			if (columns[0] == 'Instrument Type')
			{
				var instrumentType = columns[1];
				if (instrumentType == 'NovaSeq')
				{
					Wizard.setInputStatus('flowCellType', 'valid');
				}
				else
				{
					Wizard.setInputStatus('flowCellType', 'warning', 'Instrument type='+Strings.encodeTags(instrumentType)+' in the file.');
				}
			}
		}
		
		if (lineNo >= lines.length)
		{
			Wizard.setInputStatus('sampleSheetFile', 'invalid', 'Could not find [Data] section.');
			return;
		}
		
		var headers = lines[lineNo].split(/,/);
		var numHeaders = headers.length;
		
		var sampleIdIndex = seq.findColumn(headers, 'Sample_ID', lineNo);
		if (sampleIdIndex == -1) return;

		var sampleNameIndex = seq.findColumn(headers, 'Sample_Name', lineNo);
		if (sampleNameIndex == -1) return;
		
		var barcodeIndex = seq.findColumn(headers, 'I7_Index_ID', lineNo);
		if (barcodeIndex == -1) return;
		
		var idx1 = seq.findColumn(headers, 'index', lineNo);
		if (idx1 == -1) return;

		var idx2 = seq.findColumn(headers, 'index2', lineNo);
		if (idx2 == -1) return;

		var plateIndex = seq.findColumn(headers, 'Sample_Plate', lineNo, true);
		var wellIndex = seq.findColumn(headers, 'Sample_Well', lineNo, true);
		
		lineNo++;
		var libs = [];
		while (lineNo < lines.length)
		{
			var columns = lines[lineNo].split(/,/);
			if (numHeaders == columns.length)
			{
				libs[libs.length] = { 
						'lineNo': lineNo, 
						'sampleId': columns[sampleIdIndex], 
						'sampleName': columns[sampleNameIndex], 
						'barcode': columns[barcodeIndex],
						'index1': columns[idx1], 
						'index2': columns[idx2],
						'plate': plateIndex >= 0 ? columns[plateIndex] : null,
						'well': wellIndex >= 0 ? columns[wellIndex] : null
					};
			}
			lineNo++;
		}
	
		var submitInfo = {};
		submitInfo.libraries = libs;
		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=CheckExternalSequencing';
		Wizard.setInputStatus('sampleSheetFile', 'checking', 'Checking...');
		Wizard.showLoadingAnimation('Checking sample sheet...');
		Wizard.asyncJsonRequest(url, seq.sampleSheetChecked, 'POST', JSON.stringify(submitInfo));
	}
	
	// Find a column with the given header
	seq.findColumn = function(columns, header, lineNo, optional)
	{
		var colIndex = columns.indexOf(header);
		if (colIndex == -1 && !optional)
		{
			Wizard.setInputStatus('sampleSheetFile', 'invalid', 'On line '+lineNo+'; Can\'t find "' + header + '" column.');
		}
		return colIndex;
	}
	
	seq.sampleSheetChecked = function(response)
	{
		Wizard.setInputStatus('sampleSheetFile');
		sampleSheetIsValid = false;

		Wizard.showFinalMessage(response.messages);
		
		if (response.numErrors > 0)
		{
			Wizard.setInputStatus('sampleSheetFile', 'invalid', 'The sample sheet has ' + response.numErrors + ' errors.');
		}
		else
		{
			Wizard.setInputStatus('sampleSheetFile', 'valid');
			checkedLibraries = response.libraries;
			sampleSheetIsValid = true;
		}
	}
	
	seq.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		frm.clusters.length = 0;
		var clusters = response.hosts;
		if (clusters.length == 0)
		{
			Doc.element('autoAnalyzeNo').checked = true;
			autoAnalyzeIsValid = true;
			Wizard.setInputStatus('autoAnalyze', 'warning', 'No Open Grid cluster available');
			return;
		}
		
		Doc.element('autoAnalyzeNo').disabled = false;
		Doc.element('autoAnalyzeYes').disabled = false;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		seq.autoAnalyzeOnChange();
	}
	
	seq.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		frm.priority.length = 0;
		if (cluster && cluster.priorities && cluster.priorities.length > 0)
		{
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
		}
		else
		{
			frm.priority[0] = new Option('default');
		}
	}

	seq.autoAnalyzeOnChange = function()
	{
		Wizard.setInputStatus('autoAnalyze');
		autoAnalyzeIsValid = false;
		
		var frm = document.forms['reggie'];
		var yesChecked = Doc.element('autoAnalyzeYes').checked;
		var noChecked = Doc.element('autoAnalyzeNo').checked;
		
		frm.clusters.disabled = !yesChecked;
		frm.priority.disabled = !yesChecked;
		frm.debug.disabled = !yesChecked;

		if (!yesChecked && !noChecked)
		{
			Wizard.setInputStatus('autoAnalyze', 'invalid', 'Must select Yes or No');
			return;
		}
		
		autoAnalyzeIsValid = true;
		Wizard.setInputStatus('autoAnalyze', 'valid');
		
		if (yesChecked) seq.clusterOnChange();
	}

	
	seq.validateStep1 = function(event)
	{
		var valid = flowCellIdIsValid;
		valid &= sampleSheetIsValid;
		valid &= Wizard.isValid('startDate');
		valid &= autoAnalyzeIsValid;
	
		if (!valid) event.preventDefault();
	}

	seq.submit = function()
	{
		var frm = document.forms['reggie'];
		Wizard.hideWizardStatus();
		
		var submitInfo = {};
		
		submitInfo.libraries = checkedLibraries;
		submitInfo.operator = frm.operator.value;
		submitInfo.libProtocol = parseInt(frm.libProtocol.value, 10);
		submitInfo.flowCellId = frm.flowCellId.value;
		submitInfo.startDate = frm.startDate.value;
		submitInfo.flowCellType = frm.flowCellType.value;
		submitInfo.numLanes = parseInt(frm.num_lanes.value);
		submitInfo.comments = frm.comments.value;
		
		if (Doc.element('autoAnalyzeYes').checked)
		{
			var autoAnalyze = {};
			autoAnalyze.cluster = frm.clusters.value;
			autoAnalyze.priority = parseInt(frm.priority.value);
			autoAnalyze.debug = frm.debug.checked;
			submitInfo.autoAnalyze = autoAnalyze;
		}

		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterExternalSequencing';
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, seq.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	seq.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	seq.initDebugTools = function(response)
	{
		var bioplates = response.bioplates;
		var frm = document.forms['reggie'];	
		var plates = frm.libplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				plates.options[plates.length] = new Option(bioplate.name + ' (' + bioplate.usedWells + ' libraries)', bioplate.id);
			}
			
			Buttons.addClickHandler('btnFakeSampleSheet', seq.generateFakeSampleSheet);
			Wizard.showDebugTools();
		}
	}
	
	seq.generateFakeSampleSheet = function()
	{
		var frm = document.forms['reggie'];	
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=GenerateFakeSampleSheet';
		url += '&plateId='+frm.libplate.value;
		
		Wizard.showLoadingAnimation('Generating sample sheet...');
		Wizard.asyncJsonRequest(url, seq.sampleSheetGenerated);
	}

	seq.sampleSheetGenerated = function(response)
	{
		seq.parseSampleSheet(response.sampleSheet);
	}
	
	return seq;
}();

Doc.onLoad(Sequencing.initPage);

